@@ -104,16 +104,16 @@ def get_sidecar_file(
     filepath: pathlib.Path,
     relative_filepath: str | None,
     sidecar: bool,
-    sidecar_template: str | None,
+    sidecar_filename_template: str | None,
     exiftool_path: str | None,
     verbose: Callable[..., None],
 ) -> pathlib.Path | None:
-    if sidecar or sidecar_template:
-        if sidecar_template:
+    if sidecar or sidecar_filename_template:
+        if sidecar_filename_template:
             if sidecar_file := render_photo_template(
                 filepath,
                 relative_filepath,
-                sidecar_template,
+                sidecar_filename_template,
                 exiftool_path,
                 None,
             ):
@@ -728,7 +728,7 @@ def check_templates_and_exit(
     parse_date: str | None,
     parse_folder_date: str | None,
     sidecar: bool,
-    sidecar_template: str | None = None,
+    sidecar_filename_template: str | None = None,
 ):
     """Renders templates against each file so user can verify correctness"""
     for file in files:
@@ -738,7 +738,7 @@ def check_templates_and_exit(
             filepath=file,
             relative_filepath=relative_filepath,
             sidecar=sidecar,
-            sidecar_template=sidecar_template,
+            sidecar_filename_template=sidecar_filename_template,
             exiftool_path=exiftool_path,
             verbose=echo,
         )
@@ -876,7 +876,7 @@ def update_report_record(
     report_record.title = photo.title
     report_record.description = photo.description
     report_record.keywords = photo.keywords
-    report_record.location = photo.location
+    report_record.location = photo.location or (None, None)
     report_record.datetime = photo.date
 
     return report_record
@@ -1042,7 +1042,9 @@ def write_sqlite_report(
                 report_record.description,
                 ",".join(report_record.keywords),
                 ",".join(report_record.albums),
-                f"{report_record.location[0]},{report_record.location[1]}",
+                f"{report_record.location[0]},{report_record.location[1]}"
+                if report_record.location and report_record.location is not None
+                else None,
                 report_record.datetime.isoformat() if report_record.datetime else None,
             ),
         )
@@ -1153,7 +1155,7 @@ def import_files(
     exiftool_path: str,
     sidecar: bool,
     sidecar_ignore_date: bool,
-    sidecar_template: str,
+    sidecar_filename_template: str,
     merge_keywords: bool,
     title: str | None,
     description: str | None,
@@ -1215,12 +1217,12 @@ def import_files(
             )
             report_record = report_data[filepath]
 
-            if sidecar or sidecar_template:
+            if sidecar or sidecar_filename_template:
                 sidecar_file = get_sidecar_file(
                     filepath=filepath,
                     relative_filepath=relative_filepath,
                     sidecar=sidecar,
-                    sidecar_template=sidecar_template,
+                    sidecar_filename_template=sidecar_filename_template,
                     exiftool_path=exiftool_path,
                     verbose=verbose,
                 )
@@ -1766,7 +1768,7 @@ class ImportCommand(click.Command):
     "--merge-keywords",
     "-m",
     is_flag=True,
-    help="Merge keywords created by --exiftool, --sidecar, --sidecar-template, or --keyword "
+    help="Merge keywords created by --exiftool, --sidecar, --sidecar-filename, or --keyword "
     "with any keywords already associated with the photo. "
     "Without --merge-keywords, existing keywords will be overwritten.",
 )
@@ -1845,7 +1847,7 @@ class ImportCommand(click.Command):
     help="Use third party tool exiftool (https://exiftool.org/) to automatically "
     "update metadata (title, description, keywords, location) in imported photos from "
     "the imported file's metadata. "
-    "See also --sidecar, --sidecar-template. "
+    "See also --sidecar, --sidecar-filename. "
     "Note: importing keywords from video files is not currently supported.",
 )
 @click.option(
@@ -1866,13 +1868,14 @@ class ImportCommand(click.Command):
     "Supported sidecar formats are XMP and JSON (as generated by exiftool). "
     "If both JSON and XMP sidecars are found, the JSON sidecar will be used. "
     "If sidecar format is XMP, exiftool must be installed as it is used to read the XMP files. "
-    "See also --sidecar-template if you need control over the sidecar name. "
+    "See also --sidecar-filename if you need control over the sidecar name. "
     "See also --sidecar-ignore-date. "
-    "Note: --sidecar and --sidecar-template are mutually exclusive.",
+    "Note: --sidecar and --sidecar-filename are mutually exclusive.",
 )
 @click.option(
-    "--sidecar-template",
+    "--sidecar-filename",
     "-T",
+    "sidecar_filename_template",
     metavar="TEMPLATE",
     type=TemplateString(),
     help="Use sidecar files to import metadata (title, description, keywords, location). "
@@ -1885,7 +1888,7 @@ class ImportCommand(click.Command):
     "If the sidecar format is XMP, exiftool must be installed as it is used to read the XMP files. "
     "See Template System in help for additional information. "
     "See also --sidecar-ignore-date. "
-    "Note: --sidecar and --sidecar-template are mutually exclusive.",
+    "Note: --sidecar and --sidecar-filename are mutually exclusive.",
 )
 @click.option(
     "--sidecar-ignore-date",
@@ -1893,7 +1896,7 @@ class ImportCommand(click.Command):
     is_flag=True,
     help="Do not use date in sidecar to set photo date/time. "
     "Setting the timezone from sidecar files is not currently supported so when using --sidecar "
-    "or --sidecar-template, the date/time found in the sidecar will be converted to the local timezone "
+    "or --sidecar-filename, the date/time found in the sidecar will be converted to the local timezone "
     "and that value will be used to set the photo date/time. "
     "If your photos have correct timezone information in the embedded metadata you can use "
     "--sidecar-ignore-date to ignore the date/time in the sidecar and use the date/time from the "
@@ -2079,7 +2082,7 @@ def import_main(
     resume: bool,
     sidecar: bool,
     sidecar_ignore_date: bool,
-    sidecar_template: str | None,
+    sidecar_filename_template: str | None,
     skip_dups: bool,
     split_folder: str | None,
     theme: str | None,
@@ -2136,7 +2139,7 @@ def import_cli(
     resume: bool = False,
     sidecar: bool = False,
     sidecar_ignore_date: bool = False,
-    sidecar_template: str | None = None,
+    sidecar_filename_template: str | None = None,
     skip_dups: bool = False,
     split_folder: str | None = None,
     theme: str | None = None,
@@ -2177,7 +2180,7 @@ def import_cli(
             parse_date=parse_date,
             parse_folder_date=parse_folder_date,
             sidecar=sidecar,
-            sidecar_template=sidecar_template,
+            sidecar_filename_template=sidecar_filename_template,
         )
 
     # need to get the library path to initialize FingerprintQuery
@@ -2204,15 +2207,15 @@ def import_cli(
             rich_echo_error(f"[error] {e}")
             raise click.Abort()
 
-    if sidecar and sidecar_template:
+    if sidecar and sidecar_filename_template:
         rich_echo_error(
-            "[error] Only one of --sidecar or --sidecar-template may be used"
+            "[error] Only one of --sidecar or --sidecar-filename may be used"
         )
         raise click.Abort()
 
-    if sidecar_ignore_date and not (sidecar or sidecar_template):
+    if sidecar_ignore_date and not (sidecar or sidecar_filename_template):
         rich_echo_error(
-            "[error] --sidecar-ignore-date must be used with --sidecar or --sidecar-template"
+            "[error] --sidecar-ignore-date must be used with --sidecar or --sidecar-filename"
         )
         raise click.Abort()
 
@@ -2245,7 +2248,7 @@ def import_cli(
         exiftool_path=exiftool_path,
         sidecar=sidecar,
         sidecar_ignore_date=sidecar_ignore_date,
-        sidecar_template=sidecar_template,
+        sidecar_filename_template=sidecar_filename_template,
         merge_keywords=merge_keywords,
         title=title,
         description=description,
